import os
import sys

import shlex
import subprocess

import copy
import re
                    
######################################################################################
######################################################################################
######################################################################################
# CLASS PROJECT
class Project(object):
    _console = None
#    _project_component_keys = ['name', 'directory', 'binary', 'arguments', 'histolength', 'tracelength', 'all', 'depth', 'functions']
    _project_component_keys = ['name', 'directory', 'binary', 'arguments', 'histolength', 'tracelength', 'dump', 'depth', 'floatonly', 'functions', 'callids']
    #    _ilpcommandfile = "_ilptool.command"
    _ilpfiles = {"command": "_ilptool.command",
                 "xmlcommand": "_ilptool_command.xml",
                 "profile": "_ilptool.profile",
                 "histogram": "_ilptool.histogram",
                 "trace": "_ilptool.trace"
                 }
    def __init__(self, console):
        self._console = console

        self._project_components = {}

        for key in self._project_component_keys:
            self._project_components[key] = ""
        self._project_components['name'] = "New project"
        self._project_components['dump'] = "main"
        self._project_components['depth'] = "1"
        self._project_components['functions'] = "main"
        self._project_components['histolength'] = "0"
        self._project_components['tracelength'] = "0"
        self._project_components['floatonly'] = "0"
        self._project_components['functions'] = ""
        self._project_components['callids'] = ""

        
    def Save(self):
        try:
            filename = self._project_components['name']+".project"
            f = open(filename,'w')
            for key in self._project_component_keys:
                f.write(self._project_components[key])
                f.write("\n")
            self._console.Message("Writing project... Ok")
        except IOError:
            self._console.Message("IOError: impossible to save project")
            #print "Internal IOError: impossible to save project"
            #os.abort()

    def Open(self, name=None):
        try:
            filename = name
            f = open(filename,'r')
            for key in self._project_component_keys:
                self._project_components[key] = f.readline().strip()
            #self._console.Message("Reading project... Ok")
        except IOError:
            self._console.Message("Sorry, can not open project file")

    def BuildPinCommand(self, pinpath="", pinoptions="", pintoolpath=""):
        pincmd = pinpath + os.sep + "pin " + pinoptions+" -t " + pintoolpath+os.sep + "ilpnowrip -- "
        #pincmd = pincmd + os.curdir + os.sep + self._project_binary + " " + self._project_arguments
        pincmd = pincmd+os.curdir+os.sep + self._project_components['binary'] + " " + self._project_components['arguments']
        return pincmd
    
#     def GetDummyProfile(self, pinpath="", pinoptions="", pintoolpath=""):
#         pincmd = self.BuildPinCommand(pinpath, pinoptions, pintoolpath)
#         print "[DEBUG DUMMY PROFILE]"
#         print pincmd
        
    def GetProfile(self, pinpath="", pinoptions="", pintoolpath="", pinresdir=""):
        try:            
            f = open(self._ilpfiles['command'],'w')
            f.write("PROFILE")
            f.write("\n")
            f.write("0")
            f.write("\n")
            f.write(pinresdir)
            f.write("\n")
#            f.write(self._project_components['dump'])
            f.write("ALL_OFF")
            f.write("\n")
            f.write(self._project_components['depth'])
            f.write("\n")
            func_arg = self._project_components['functions'].split()
            #print func_arg
            for func in func_arg:
                f.write(func)
                f.write("\n")
            f.close()

            f = open(self._ilpfiles['xmlcommand'],'w')
            f.write('<ilpcommand>\n')

            f.write('  <command>profile</command>\n')

            f.write('  <resdir>')
            f.write(pinresdir)
            f.write('</resdir>\n')

            f.write('  <inst_control>all')
            f.write('</inst_control>\n')

            f.write('  <dump_control>')
            f.write(self._project_components['dump'])
            f.write('</dump_control>\n')
            
            f.write('  <depth>')
            f.write(self._project_components['depth'])
            f.write('</depth>\n')

            f.write('  <length>0')
            f.write('</length>\n')

            f.write('  <floatonly>')
            f.write(self._project_components['floatonly'])            
            f.write('</floatonly>\n')

            f.write('</ilpcommand>\n')

            f.close()

#             f.write("PROFILE")
#             f.write("\n")
#             f.write("0")
#             f.write("\n")
#             f.write(pinresdir)
#             f.write("\n")
#             f.write(self._project_components['all'])
#             f.write("\n")
#             f.write(self._project_components['depth'])
#             f.write("\n")
#             func_arg = self._project_components['functions'].split()
#             #print func_arg
#             for func in func_arg:
#                 f.write(func)
#                 f.write("\n")

            print 
            pincmd = self.BuildPinCommand(pinpath, pinoptions, pintoolpath)
            pinargs = shlex.split(pincmd)
            print pincmd
            #self._console.Message(pincmd)
            #             print "[DEBUG]",
            #             print pinargs
            #            res = subprocess.Popen(pinargs)
            res = subprocess.call(pinargs)
            #os.execv(pincmd)



        except IOError:
            self._console.Message("IOError: impossible to write ilp command file")
            #        os.chdir(self._setup._absilpdir)
            
        # Auto set histo lenght and trace lenght
        # ========================================
        filename = pinresdir+os.sep+self._ilpfiles['profile']
        fin = open(filename,'r')
        glb_pattern = "Global ILP.*I\[(\d+)\]::C\[(\d+)\]::ILP\[(\d+\.\d*)\].*"
        print 
        while True:
            line = fin.readline()
            if not line:
                break
            match = re.match(glb_pattern, line)
            if match != None:
                # print "HistoL found ", match.group(2)
                # print "TraceL found ", match.group(1)
                self._project_components['histolength'] = str(int(match.group(2))+1)
                self._project_components['tracelength'] = str(int(match.group(1))+1)
                # self.Save()
                break
        fin.close()
        return

    def GetHistogram(self, pinpath="", pinoptions="", pintoolpath="", pinresdir=""):
        try:            
            f = open(self._ilpfiles['command'],'w')
            f.write("HISTOGRAM")
            f.write("\n")
            f.write(self._project_components['histolength'])
            f.write("\n")
            f.write(pinresdir)
            f.write("\n")
            #f.write(self._project_components['all'])
            f.write("ALL_OFF")
            f.write("\n")
            f.write(self._project_components['depth'])
            f.write("\n")
            func_arg = self._project_components['functions'].split()
            #print func_arg
            for func in func_arg:
                f.write(func)
                f.write("\n")
            f.close()

            f = open(self._ilpfiles['xmlcommand'],'w')
            f.write('<ilpcommand>\n')

            f.write('  <command>histogram</command>\n')

            f.write('  <resdir>')
            f.write(pinresdir)
            f.write('</resdir>\n')

            f.write('  <inst_control>all')
            f.write('</inst_control>\n')

            f.write('  <dump_control>')
            f.write(self._project_components['dump'])
            f.write('</dump_control>\n')
            
            f.write('  <depth>')
            f.write(self._project_components['depth'])
            f.write('</depth>\n')

            f.write('  <length>')
            f.write(self._project_components['histolength'])
            f.write('</length>\n')

            f.write('  <floatonly>')
            f.write(self._project_components['floatonly'])            
            f.write('</floatonly>\n')

            f.write('</ilpcommand>\n')

            f.close()
            
            pincmd = self.BuildPinCommand(pinpath, pinoptions, pintoolpath)
            pinargs = shlex.split(pincmd)
            print pincmd
            #self._console.Message(pincmd)
            #             print "[DEBUG]",
            #             print pinargs
            #            res = subprocess.Popen(pinargs)
            res = subprocess.call(pinargs)
            #os.execv(pincmd)
        except IOError:
            self._console.Message("IOError: impossible to write ilp command file")
            #        os.chdir(self._setup._absilpdir)
        return

    def GetShortHistogram(self, pinpath="", pinoptions="", pintoolpath="", pinresdir=""):
        try:            
            f = open(self._ilpfiles['command'],'w')
            f.write("SHORTHISTOGRAM")
            f.write("\n")
            f.write(self._project_components['histolength'])
            f.write("\n")
            f.write(pinresdir)
            f.write("\n")
            #f.write(self._project_components['all'])
            f.write("ALL_OFF")
            f.write("\n")
            f.write(self._project_components['depth'])
            f.write("\n")
            func_arg = self._project_components['functions'].split()
            #print func_arg
            for func in func_arg:
                f.write(func)
                f.write("\n")
            f.close()
            
            pincmd = self.BuildPinCommand(pinpath, pinoptions, pintoolpath)
            pinargs = shlex.split(pincmd)
            print pincmd
            #self._console.Message(pincmd)
            #             print "[DEBUG]",
            #             print pinargs
            #            res = subprocess.Popen(pinargs)
            res = subprocess.call(pinargs)
            #os.execv(pincmd)
        except IOError:
            self._console.Message("IOError: impossible to write ilp command file")
            #        os.chdir(self._setup._absilpdir)
        return

#     def GetHistogram(self, pinpath="", pinoptions="", pintoolpath=""):
#         try:
#             f = open(self._ilpcommandfile,'w')
#             f.write("HISTOGRAM")
#             #            f.write(histogram_length)
#             f.write(self._project_components['histolength'])
#             f.write("\n")
            
#             pincmd = self.BuildPinCommand(pinpath, pinoptions, pintoolpath)
#             #os.execv(pincmd)
#         except IOError:
#             self._console.Message("IOError: impossible to write ilp command file")
#         return        

    def GetTrace(self, pinpath="", pinoptions="", pintoolpath="", pinresdir=""):
#         try:
#             os.chdir(self._project_components['directory'])
#         except OSError:
#             self._console.Message("Error: can not access project directory")
#             return
        try:            
            f = open(self._ilpfiles['command'],'w')
            f.write("TRACE")
            f.write("\n")
            f.write(self._project_components['tracelength'])
            f.write("\n")
            f.write(pinresdir)
            f.write("\n")
            #f.write(self._project_components['all'])
            f.write("ALL_OFF")
            f.write("\n")
            f.write(self._project_components['depth'])
            f.write("\n")
            func_arg = self._project_components['functions'].split()
            #print func_arg
            for func in func_arg:
                f.write(func)
                f.write("\n")
            f.close()
           
            f = open(self._ilpfiles['xmlcommand'],'w')
            f.write('<ilpcommand>\n')

            f.write('  <command>trace</command>\n')

            f.write('  <resdir>')
            f.write(pinresdir)
            f.write('</resdir>\n')

            f.write('  <inst_control>all')
            f.write('</inst_control>\n')

            f.write('  <dump_control>')
            f.write(self._project_components['dump'])
            f.write('</dump_control>\n')
            
            f.write('  <depth>')
            f.write(self._project_components['depth'])
            f.write('</depth>\n')

            f.write('  <length>')
            f.write(self._project_components['tracelength'])
            f.write('</length>\n')

            f.write('  <floatonly>')
            f.write(self._project_components['floatonly'])            
            f.write('</floatonly>\n')

            f.write('</ilpcommand>\n')

            f.close()

            pincmd = self.BuildPinCommand(pinpath, pinoptions, pintoolpath)
            pinargs = shlex.split(pincmd)
            print pincmd
            #self._console.Message(pincmd)
            #             print "[DEBUG]",
            #             print pinargs
            #            res = subprocess.Popen(pinargs)
            res = subprocess.call(pinargs)
            #os.execv(pincmd)
        except IOError:
            self._console.Message("IOError: impossible to write ilp command file")
            #        os.chdir(self._setup._absilpdir)
        return

#     def GetTrace(self, pinpath="", pinoptions="", pintoolpath=""):
#         try:
#             f = open(self._ilpcommandfile,'w')
#             f.write("TRACE")
#             #            f.write(trace_length)
#             f.write(self._project_components['tracelength'])
#             f.write("\n")
            
#             pincmd = self.BuildPinCommand(pinpath, pinoptions, pintoolpath)
#             #os.execv(pincmd)
#         except IOError:
#             self._console.Message("IOError: impossible to write ilp command file")
#         return
        
######################################################################################
######################################################################################
######################################################################################
# CLASS PROJECT MANAGER
class ProjectManager:
    _console = None
    _setup = None
    _project_list = []
    def __init__(self,cons=None,setup=None):
        if (cons == None):
            print "Internal Error console is needed"
            os.abort()
        self._console = cons    
        if (setup == None):
            print "Internal Error setup is needed"
            os.abort()
        self._setup = setup
            
        return
    
    def Create(self):
        self._project_list.append(copy.copy(Project(self._console)))
        return (len(self._project_list)-1)
    
    def Open(self,name=None):
        self._project_list.append(copy.copy(Project(self._console)))
        self._project_list[len(self._project_list)-1].Open(name)
        return

    def Save(self, pnum=None):
        if (pnum != None):
            if (not(os.access(self._project_list[pnum]._project_components['name'], os.F_OK))):
                os.mkdir(self._project_list[pnum]._project_components['name'])
            os.chdir(self._project_list[pnum]._project_components['name'])
            self._project_list[pnum].Save()
            os.chdir(os.pardir)

    def Remove(self, pnum=None):
        if (pnum != None):
            self._project_list.pop(pnum)
            
    def HomeDir(self,pnum):
        home_dir = self._setup._absilpdir + os.sep + self._setup._project_directory
        home_dir = home_dir + os.sep + self._project_list[pnum]._project_components['name'] + os.sep
        return home_dir

    def Profile(self, pnum=None):
        if (pnum != None):
            try:
                os.chdir(self._project_list[pnum]._project_components['directory'])
                print os.getcwd()
            except OSError:
                self._console.Message("Error: can not access directory :")
                self._console.Message(self._project_list[pnum]._project_components['directory'])
                return

            home_dir = self.HomeDir(pnum)

            self._project_list[pnum].GetProfile(self._setup._pinpath,
                                                self._setup._pinoptions,
                                                self._setup._pintoolpath,
                                                home_dir)
            #             # Move command file
            movecmd = "mv -f "+self._project_list[pnum]._ilpfiles['command']+" "+home_dir
            print movecmd        
            moveargs = shlex.split(movecmd)
            res = subprocess.call(moveargs)
            #             # Move xml command file
            movecmd = "mv -f "+self._project_list[pnum]._ilpfiles['xmlcommand']+" "+home_dir
            print movecmd        
            moveargs = shlex.split(movecmd)
            res = subprocess.call(moveargs)
            os.chdir(self._setup._absilpdir)
            os.chdir(self._setup._project_directory)

    def Histogram(self, pnum=None):
        if (pnum != None):
            try:
                os.chdir(self._project_list[pnum]._project_components['directory'])
            except OSError:
                self._console.Message("Error: can not access directory")
                self._console.Message(self._project_list[pnum]._project_components['directory'])
                return
            home_dir = self.HomeDir(pnum)
            self._project_list[pnum].GetHistogram(self._setup._pinpath,
                                                  self._setup._pinoptions,
                                                  self._setup._pintoolpath,
                                                  home_dir)
            #             # Move command file
            movecmd = "mv -f "+self._project_list[pnum]._ilpfiles['command']+" "+home_dir
            print movecmd        
            moveargs = shlex.split(movecmd)
            res = subprocess.call(moveargs)            
             #             # Move xml command file
            movecmd = "mv -f "+self._project_list[pnum]._ilpfiles['xmlcommand']+" "+home_dir
            print movecmd        
            moveargs = shlex.split(movecmd)
            res = subprocess.call(moveargs)
            os.chdir(self._setup._absilpdir)
            os.chdir(self._setup._project_directory)

    def ShortHistogram(self, pnum=None):
        if (pnum != None):
            try:
                os.chdir(self._project_list[pnum]._project_components['directory'])
            except OSError:
                self._console.Message("Error: can not access directory")
                self._console.Message(self._project_list[pnum]._project_components['directory'])
                return
            home_dir = self.HomeDir(pnum)
            self._project_list[pnum].GetShortHistogram(self._setup._pinpath,
                                                       self._setup._pinoptions,
                                                       self._setup._pintoolpath,
                                                       home_dir)
            #             # Move command file
            movecmd = "mv -f "+self._project_list[pnum]._ilpfiles['command']+" "+home_dir
            print movecmd        
            moveargs = shlex.split(movecmd)
            res = subprocess.call(moveargs)            
             #             # Move xml command file
            movecmd = "mv -f "+self._project_list[pnum]._ilpfiles['xmlcommand']+" "+home_dir
            print movecmd        
            moveargs = shlex.split(movecmd)
            res = subprocess.call(moveargs)
            os.chdir(self._setup._absilpdir)
            os.chdir(self._setup._project_directory)

    def Trace(self, pnum=None):
        if (pnum != None):
            try:
                os.chdir(self._project_list[pnum]._project_components['directory'])
            except OSError:
                self._console.Message("Error: can not access directory")
                self._console.Message(self._project_list[pnum]._project_components['directory'])
                return
            home_dir = self.HomeDir(pnum)
            self._project_list[pnum].GetTrace(self._setup._pinpath,
                                              self._setup._pinoptions,
                                              self._setup._pintoolpath,
                                              home_dir)
            #             # Move command file
            movecmd = "mv -f "+self._project_list[pnum]._ilpfiles['command']+" "+home_dir
            print movecmd        
            moveargs = shlex.split(movecmd)
            res = subprocess.call(moveargs)
             #             # Move xml command file
            movecmd = "mv -f "+self._project_list[pnum]._ilpfiles['xmlcommand']+" "+home_dir
            print movecmd        
            moveargs = shlex.split(movecmd)
            res = subprocess.call(moveargs)
            os.chdir(self._setup._absilpdir)
            os.chdir(self._setup._project_directory)

    def Clean(self, pnum=None):
        if (pnum != None):
            try:
                os.chdir(self._project_list[pnum]._project_components['directory'])
            except OSError:
                self._console.Message("Error: can not access directory")
                self._console.Message(self._project_list[pnum]._project_components['directory'])
                return
            home_dir = self.HomeDir(pnum)
            #             # Move command file
            #            print os.getcwd()
            #            print rmcmd        
            print "[Cleaning...]  ", home_dir

            rmcmd = "rm -f "+home_dir+os.sep+"*.profile"
            res = subprocess.call(rmcmd, shell=True)
            print rmcmd        

            rmcmd = "rm -f "+home_dir+os.sep+"*.histogram"
            res = subprocess.call(rmcmd, shell=True)
            print rmcmd        

            rmcmd = "rm -f "+home_dir+os.sep+"*.shorthistogram"
            res = subprocess.call(rmcmd, shell=True)
            print rmcmd        

            rmcmd = "rm -f "+home_dir+os.sep+"*.trace"
            res = subprocess.call(rmcmd, shell=True)
            print rmcmd        

            rmcmd = "rm -f "+home_dir+os.sep+"*.gootrace"
            res = subprocess.call(rmcmd, shell=True)
            print rmcmd        

            rmcmd = "rm -f "+home_dir+os.sep+"*.colmap"
            res = subprocess.call(rmcmd, shell=True)
            print rmcmd        

            os.chdir(self._setup._absilpdir)
            os.chdir(self._setup._project_directory)

    def Quit(self):
        namelist = []
        for proj in self._project_list:
            fullname = os.getcwd() + os.sep + proj._project_components['name'] + os.sep + proj._project_components['name'] + '.project'
            #            print fullname
            namelist.append(fullname)
        self._setup.Quit(namelist)
        
